#!coding:utf-8


def transitions_to_dotlang(matrix):
    dim = len(matrix)
    transitions = []
    for current_state in range(dim):
        for next_state in range(dim):
            t = '   %s -- %s [label = "%.4f"]' % (current_state, next_state, matrix[current_state][next_state])
            transitions.append(t)

    return "digraph G {\n%s\n}" % ('\n'.join(transitions))


def transitions_to_graph(digraph, matrix, state_index_to_name = lambda s: "%s" % s):
    dim = len(matrix)
    for i in range(dim):
        digraph.node(state_index_to_name(i))
    for current_state in range(dim):
        for next_state in range(dim):
            alpha = "{0:x}".format(int(round(matrix[current_state][next_state] * 250 + 6)))
            digraph.edge(state_index_to_name(current_state),
                         state_index_to_name(next_state),
                         label=" %.4f " % matrix[current_state][next_state],
                         color='#ff0000%s' % alpha,
                         fontcolor='#f00000%s' % alpha)

if __name__ == '__main__':
    from graphviz import Digraph

    matrix = [[0.5, 0.5], [0.5, 0.5]]
    print transitions_to_dotlang(matrix)
    g = Digraph(format='svg', engine='dot',
                graph_attr={'overlap': 'true', 'pad': '0.1', 'nodesep': '0.5', 'ranksep': '1.0'},
                edge_attr={'fontname':'HelveticaNeue'})

    transitions_to_graph(g, matrix, lambda s: "heads" if s == 1 else "tails")
    g.render('./graph')



from copy import copy
from random import random


class Chain(object):
    def __init__(self, states_count, initial_state):
        self._states_count = states_count
        self._initial_state = initial_state
        self.clear()

    """
        Build Markov chain from data by feeding new state of the random process
    """
    def learn(self, state):
        self.states[self.current_state][state] = self.states[self.current_state][state] + 1
        self.current_state = state

    """
        Get the transition matrix P of Markov chain
    """
    def transition_matrix(self):
        m = copy(self.states)
        for i in range(self._states_count):
            s = 1.0 / sum([m[i][j] for j in range(self._states_count)])
            m[i] = [m[i][j] * s for j in range(self._states_count)]
        return m

    """
        Get probabilities of transitions to another states from current state
    """
    def get_transitions_probs(self, state):
        s = 1.0 / sum([self.states[state][j] for j in range(self._states_count)])
        c = copy(self.states[state])
        return [c[i] * s for i in range(self._states_count)]

    """
        Clear the model
    """
    def clear(self):
        self.states = [[0 for j in range(self._states_count)] for i in range(self._states_count)]
        self.current_state = self._initial_state



class FSM(object):
    def __init__(self, chain, initial_state, rnd=random):
        self.chain = chain
        self.current_state = initial_state
        self._rnd = rnd

    @property
    def state(self):
        return self.current_state

    def next(self):
        probs = self.chain.get_transitions_probs(self.current_state)
        r = self._rnd()

        p0 = 0.0
        p1 = 0.0

        for i in range(len(probs)):
            p1 = p0 + probs[i]
            if r >= p0 and r < p1:
                self.current_state = i
                return self.current_state
            else:
                p0 = p1



if __name__ == '__main__':
    from random import random

    # build Markov chain with 2 states, init with random state
    chain = Chain(2, 0 if random() > 0.5 else 1)

    # flip coin many times and build Markov chain for this process
    # let 0 be heads and 1 tails
    for i in range(1000000):
        chain.learn(0 if random() > 0.5 else 1)

    # get transition matrix
    #   It should look like:
    #
    #    P = | 0.5 0.5 |
    #        | 0.5 0.5 |
    #
    P = chain.transition_matrix()

    print "%s %s" % (P[0][0], P[0][1])
    print "%s %s" % (P[1][0], P[1][1])

    # get probabilities of transition from state 0 to other states (0 and 1)
    # actually, the line in the transition matrix
    print chain.get_transitions_probs(0)

    # let's make a FSM with stochastic properties equal to described by Markov chain
    # use rnd() as a random numbers generator, and 0 (heads) as initial state
    fsm = FSM(chain, 0)

    # let's make another chain, build upon random process emulated by FSM
    chain2 = Chain(2, 0)

    # make a chain with states produced by FSM
    for j in range(1000000):
        chain2.learn(fsm.next())

    # get transition matrix. it should be nearly equal
    P2 = chain2.transition_matrix()

    print "%s %s" % (P2[0][0], P2[0][1])
    print "%s %s" % (P2[1][0], P2[1][1])

    print chain2.get_transitions_probs(0)



from setuptools import setup, find_packages
import sys, os

here = os.path.abspath(os.path.dirname(__file__))
README = open(os.path.join(here, 'README.md')).read()
NEWS = open(os.path.join(here, 'NEWS.txt')).read()


version = '0.2'

install_requires = []


setup(name='markovfsm',
    version=version,
    description="Markov Chain generator from random process data",
    long_description=README + '\n\n' + NEWS,
    classifiers=[
      'Development Status :: 3 - Alpha',
      'Intended Audience :: Developers',
      'Programming Language :: Python :: 2',
      'Programming Language :: Python :: 2.6',
      'Programming Language :: Python :: 2.7',
      'Programming Language :: Python :: 3',
      'Programming Language :: Python :: 3.2',
      'Programming Language :: Python :: 3.3',
      'Programming Language :: Python :: 3.4',
      'Topic :: Software Development :: Libraries',
      'Topic :: Scientific/Engineering :: Information Analysis',
      'Topic :: Scientific/Engineering :: Artificial Intelligence',
      'License :: CC0 1.0 Universal (CC0 1.0) Public Domain Dedication',
      'License :: OSI Approved :: MIT License',
      'Natural Language :: English'
    ],
    keywords='markov chain random process probability stochastic automata finite-state machine fsm',
    author='Vladimir Ignatev',
    author_email='ya.na.pochte@gmail.com',
    url='https://github.com/vladignatyev/markovfsm',
    license='MIT',
    packages=find_packages('src'),
    package_dir = {'': 'src'},include_package_data=True,
    zip_safe=False,
    install_requires=install_requires
)
